-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- P8. Tutorial on the structure of a CLK generator:
-- OSC_CLK_in = 8 MHz  (Xilinx Coolrunner board)
-- OSC_CLK_out divider by 1) --> 8 MHz  
-- CLK_1KHz_SQ, the system clock for the FSM
-- CLK_5Hz_SQ, the signal for flashing the torche 

-- Plan: Frequency dividers and T-FF to square the pulsed terminal count signals    
-- Count Enable (CE) and asynchronous active-high clear direct (CD)
-- 2 cascaded frequency dividers in series 
-- 2 T-FF to square the pulsed waveforms 
-- reference schematic: 
--------------------------------------------------------------------------------
LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;
   USE IEEE.STD_LOGIC_ARITH.ALL;
   USE IEEE.STD_LOGIC_UNSIGNED.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	CD,CE			: 	IN  std_logic;
			OSC_CLK_in		: 	IN  std_logic;
			OSC_CLK_out		:	OUT std_logic;
			CLK_1kHz_SQ		:	OUT std_logic;
			CLK_5Hz_SQ		:	OUT std_logic
          );
END CLK_Generator ;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_4000 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC4000		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_200 IS                 
		PORT(	CD,CLK,CE	: 	IN  std_logic;
				TC200		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT T_Flip_Flop IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_2kHz		:	std_logic;
	SIGNAL Pulsed_10Hz		:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_4000
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC4000	=> Pulsed_2kHz					  
		);

	Chip2	:  Freq_Div_200
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_2kHz,
			TC200 	=> Pulsed_10Hz					  
		);

	Chip3	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2kHz,
			Q 		=> CLK_1kHz_SQ				  
		);		

	Chip4	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_10Hz,
			Q 		=> CLK_5Hz_SQ				  
		);		
			
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	-- A copy of the oscillator CLK signal
	OSC_CLK_out	<= CLK;
	
END schematic ;
